@@ -187,6 +187,12 @@ static int rsync_module(int fd, int i)
 		gid = atoi(p);
 	}
 
+	p = lp_include_from(i);
+	add_exclude_file(p, 1, 1);
+
+	p = lp_include(i);
+	add_include_line(p);
+
 	p = lp_exclude_from(i);
 	add_exclude_file(p, 1, 0);
 
@@ -253,7 +259,7 @@ static int rsync_module(int fd, int i)
 				request = strdup(p);
 				start_glob++;
 			}
-			glob_expand(name, argv, &argc, MAX_ARGS);
+			glob_expand(name, argv, &argc, MAX_ARGS, !use_chroot);
 		} else {
 			argc++;
 		}
@@ -276,9 +282,7 @@ static int rsync_module(int fd, int i)
 		 *    and which aren't.
 		 */
 		for (i = 1; i < argc; i++) {
-			char *copy = sanitize_path(argv[i]);
-			free((void *)argv[i]);
-			argv[i] = copy;
+			sanitize_path(argv[i]);
 		}
 	}
 
@@ -310,8 +314,12 @@ static int rsync_module(int fd, int i)
 		io_start_multiplex_out(fd);
 
 	if (!ret) {
-		rprintf(FERROR,"Error parsing options (unsupported option?) - aborting\n");
-		exit_cleanup(RERR_SYNTAX);
+		option_error();
+	}
+
+	if (lp_timeout(i)) {
+		extern int io_timeout;
+		io_timeout = lp_timeout(i);
 	}
 
 	start_server(fd, fd, argc, argp);
